;==============================================================================================================================================================================================
;Load NetLogo extensions that are needed in the simulation process ============================================================================================================================
;==============================================================================================================================================================================================
extensions[rngs]              ;; extension for initializing pseudo random numbers

;==============================================================================================================================================================================================
;Create link breeds that indicate relations between individuals ===============================================================================================================================
;Create turtle breeds that distinguish between individuals and marriages that have been formed ================================================================================================
;==============================================================================================================================================================================================
undirected-link-breed [dating_relations dating_relation]     ;; indicates which pairs of individuals are currently dating
undirected-link-breed [marriage_relations marriage_relation] ;; indicates which pairs of individuals are currently married

breed [marriage_records marriage_record]
breed [individuals individual]

;==============================================================================================================================================================================================
;Create global and other variables ============================================================================================================================================================
;==============================================================================================================================================================================================
globals
[
  s_data_males                ;; list that stores information about the distribution of educational degrees among males in the focal country
  y_data_males_low            ;; list that stores information about the earnings prospects distribution among males with low education in the focal country
  y_data_males_high           ;; list that stores information about the earnings prospects distribution among males with high education in the focal country

  s_data_females              ;; list that stores information about the distribution of educational degrees among females in the focal country
  y_data_females_low          ;; list that stores information about the earnings prospects distribution among females with low education in the focal country
  y_data_females_high         ;; list that stores information about the earnings prospects distribution among females with high education in the focal country

  ini_share_male_data         ;; list that stores information about the share of males in the initial population
  ini_age_structure_data      ;; list that stores information about the initial age distribution in the initial population
  asfr_data                   ;; list that stores information about fertility rates
  asfr_current                ;; list that stores information about fertility rates that are relevant for the current simulation year
  d_data                      ;; list that stores information about mortality rates
  d_current                   ;; list that stores information about mortality rates that are relevant for the current simulation year

  measurement_points_ind      ;; list that stores information about the simulation years in which a census from the population of individuals should be taken
  measurement_points_mar      ;; list that stores information about the simulation years in which a census from the population of marriages should be taken

  my-seed                     ;; variable that stores information about the seed that is used for initializing pseudo random numbers
  year                        ;; variable that stores information about the simulation year in which the simulation run currently is
  ref_year                    ;; variable that stores information about the year on which the selected input data should be based

  file_name_master            ;; variable that stores the master name for all output files
  random_id                   ;; variable needed to create unique run id
  individual_file             ;; variable that stores the name of the individual-level outcome file to be created within a given run
  marriage_file               ;; variable that stores the name of the marriage-level outcome file to be created within a given run

  set_up_done                 ;; variable that indicates whether the setup has been completed and thereby allows assigning agents the correct age at the beginning of the simulation
]

individuals-own
[
  g_i                         ;; individual i's gender
  a_i                         ;; individual i's age
  s_i                         ;; individual i's ultimate educational attainment
  y_i                         ;; individual i's earnings prospects
  l_i                         ;; individual i's relationship status
  c_i                         ;; time that individual i is in its current relation (dating or in married)
  r_i                         ;; individual i's current school enrolment status

  sigma_i                     ;; auxiliary variable that stores information about the value of the age pressure parameter that applies to individual i
  beta_i                      ;; auxiliary variable that stores information about the value of the commitment parameter that applies to individual i
  ws_i                        ;; auxiliary variable that stores information about the importance that individual i attaches to the education of partners
  wy_i                        ;; auxiliary variable that stores information about the importance that individual i attaches to the earnings prospects of partners
  wa_i                        ;; auxiliary variable that stores information about the importance that individual i attaches to the age of partners
  p_seek_i                    ;; probability that individual i will actively seek out somebody in the current simulation step

  year_of_birth               ;; auxiliary variable that stores information about the simulation year in which individual i has entered the simulation
  marriage_proposal           ;; auxiliary variable that stores information about whether or not individual i has proposed marriage to its current partner

  n_relations                 ;; auxiliary variable that stores information about the number of relations an individual has been in
  n_marriages                 ;; auxiliary variable that stores information about the number of marriages an individual has been in
  n_divorces                  ;; auxiliary variable that stores information about the number divorces an individual has experienced
  n_widowed                   ;; auxiliary variable that stores information about how many of individual's spouses died
  current_marriage_id         ;; auxiliary variable that stores information about which of the marriages in the marriage  population this individual belongs to

  relation_check              ;; auxiliary variable that makes it possible to check whether a selected alternative is already the partner of individual i
]

marriage_records-own
[
  marriage_id                 ;; unique id of a marriage based on the ids of the two partners

  m_id                        ;; id of the male member of the marriage
  m_s                         ;; education of the male member of the marriage
  m_y                         ;; earnings prospects of the male member of the marriage
  m_a                         ;; age of the male member of the marriage at time of marriage

  f_id                        ;; id of the female member of the marriage
  f_s                         ;; education of the female member of the marriage
  f_y                         ;; earnings prospects of the female member of the marriage
  f_a                         ;; age of the female member of the marriage at time of marriage

  m_marriage_order            ;; order of the marriage from the male member's perspective
  f_marriage_order            ;; order of the marriage from the female member's perspective

  year_marriage               ;; simulation year in which the marriage was formed
  year_dissolution            ;; simulation year in which the marriage was divorced (if divorced)
  marriage_duration           ;; duration of the marriage in years
  dissolution_reason          ;; auxiliary variable that stores information about the reason for the dissolution (death or divorce)
  dissolution_initiator       ;; auxiliary variable that, in case one of the partner breaks up, stores information about who initiated the dissolution because of finding a more attractive alternative
]

;==============================================================================================================================================================================================
;Initialize model run =========================================================================================================================================================================
;==============================================================================================================================================================================================
to setup
  __clear-all-and-reset-ticks        ;; clear the world

  do_initialize_seed                 ;; procedure that initializes the seed that is to be used for initializing pseudo random numbers
  do_update_year                     ;; procedure that updates the information about the simulation year in which the simulation currently is/starts
  do_initialize_input_data           ;; procedure that reads the country specific input data that is necessary to generate realistic populations/cohorts
  do_initialize_individuals          ;; procedure that generates an initial population based on the input data
  do_assign_individuals_s            ;; procedure that assigns individuals their states on s
  do_assign_individuals_y            ;; procedure that assigns individuals their states on y
  do_assign_individuals_a            ;; procedure that assigns individuals their states on a
  do_update_school_enrolment_status  ;; procedure that updates the school enrolment status of all individuals

  ;Initialize output files, if necessary
  if write_to_file_individual = TRUE or write_to_file_marriage = TRUE [do_create_file_name_master]
  if write_to_file_individual = TRUE [do_initialize_individual_file]
  if write_to_file_marriage   = TRUE [do_initialize_marriage_file]

  set set_up_done TRUE
end

to do_initialize_seed
  ;Initializes a random number seed that will be used to initialize the pseudo random numbers to be used in a given simulation run
  ;============================================================================================================================================================================================
  rngs:init                                      ;; clears all random number generators

  if (seed_to_use = "random")
  [
    set my-seed new-seed                         ;; generate a new seed
    random-seed my-seed                          ;; use the new seed
  ]

  if (seed_to_use = "own")
  [
    set my-seed own_seed                         ;; get user defined seed
    random-seed my-seed                          ;; use the user defined seed
  ]

  rngs:set-seed 1 my-seed                        ;; use the selected seed to initialize the random number generator
end

to do_update_year
  ;Updates the information about the year in which the simulation model currently is.
  ;During the burn-in phase, the simulation year is 0; after this phase, the simulation commences in the starting year defined by the user (e.g., in 1921)
  ;============================================================================================================================================================================================
  ifelse (ticks / steps_per_year) < burn_in_years
  [
    set year 0
  ]
  [
    set year ((ticks / steps_per_year) - burn_in_years) + start_year
  ]

  if year < start_year [set ref_year start_year]
  if year > 2012 [set ref_year 2012]
  if year >= start_year and year <= 2012 [set ref_year year]
end

to do_initialize_input_data
  ;Reads all input files
  ;============================================================================================================================================================================================

  ;Read the input data for male individuals
  ;------------------------------------------------------------------
  let file word country "_input_male_s.txt"
  set file word "input\\" file
  file-open file
  set s_data_males file-read
  file-close

  set file word country "_input_male_y_s_low.txt"
  set file word "input\\" file
  file-open file
  set y_data_males_low file-read
  file-close

  set file word country "_input_male_y_s_high.txt"
  set file word "input\\" file
  file-open file
  set y_data_males_high file-read
  file-close

  ;Read the input data for female individuals
  ;------------------------------------------------------------------
  set file word country "_input_female_s.txt"
  set file word "input\\" file
  file-open file
  set s_data_females file-read
  file-close

  set file word country "_input_female_y_s_low.txt"
  set file word "input\\" file
  file-open file
  set y_data_females_low file-read
  file-close

  set file word country "_input_female_y_s_high.txt"
  set file word "input\\" file
  file-open file
  set y_data_females_high file-read
  file-close

  ;Read the input data for additional population characteristics
  ;------------------------------------------------------------------
  set file word country "_input_ini_age_structure.txt"
  set file word "input\\" file
  file-open file
  set ini_age_structure_data file-read
  file-close

  set file word country "_input_share_male.txt"
  set file word "input\\" file
  file-open file
  set ini_share_male_data file-read
  file-close

  set file word country "_input_asfr.txt"
  set file word "input\\" file
  file-open file
  set asfr_data file-read
  file-close

  set file word country "_input_d.txt"
  set file word "input\\" file
  file-open file
  set d_data file-read
  file-close

  ;Read the input data for the measurement points
  ;------------------------------------------------------------------
  set file "input\\measurement_points_ind.txt"
  file-open file
  set measurement_points_ind file-read
  file-close

  set file "input\\measurement_points_mar.txt"
  file-open file
  set measurement_points_mar file-read
  file-close
end

to do_initialize_individuals
  ;Creates the initial population
  ;============================================================================================================================================================================================

  ;Create male and female individuals
  ;------------------------------------------------------------------
  create-individuals round (item 0 ini_share_male_data * pop_size)
  [
    set g_i "male"
    set s_i "NA"
    set y_i "NA"
    set a_i "NA"
    set r_i "NA"
    set shape "triangle"
    set color blue
    set sigma_i sigma_m
    set beta_i beta_m
    set wa_i wa_m
    set ws_i ws_m
    set wy_i wy_m
    set c_i 0
    set l_i "single"
    set p_seek_i 0
    set year_of_birth year
    set marriage_proposal nobody
    set n_relations 0
    set n_marriages 0
    set n_divorces 0
    set n_widowed 0
    set current_marriage_id "NA"
    setxy random-xcor random-ycor
    set label who
  ]

  create-individuals round ((1 - item 0 ini_share_male_data) * pop_size)
  [
    set g_i "female"
    set s_i "NA"
    set y_i "NA"
    set a_i "NA"
    set r_i "NA"
    set shape "circle"
    set color red
    set sigma_i sigma_f
    set beta_i beta_f
    set wa_i wa_f
    set ws_i ws_f
    set wy_i wy_f
    set c_i 0
    set l_i "single"
    set p_seek_i 0
    set year_of_birth year
    set marriage_proposal nobody
    set n_relations 0
    set n_marriages 0
    set n_divorces 0
    set n_widowed 0
    set marriage_proposal nobody
    set current_marriage_id "NA"
    setxy random-xcor random-ycor
    set label who
  ]
end

to do_assign_individuals_s
  ;Assigns individuals their highest educational attainment
  ;============================================================================================================================================================================================
  ask individuals with [s_i = "NA"]
  [
    let ref_dat "NA"

    ifelse g_i = "male"
    [
      set ref_dat s_data_males
    ]
    [
      set ref_dat s_data_females
    ]

    let column 1
    let lower_boundary 0
    let upper_boundary lower_boundary + item 1 first filter [first ? = ref_year] ref_dat
    let pick random-float (sum but-first first filter [first ? = ref_year] ref_dat)

    while [s_i = "NA"]
    [
      ifelse (pick >= lower_boundary and pick < upper_boundary)
      [
        set s_i column
      ]
      [
        set column column + 1
        set lower_boundary upper_boundary
        set upper_boundary upper_boundary + item column first filter [first ? = ref_year] ref_dat
      ]
    ]
  ]
end

to do_assign_individuals_y
  ;Assigns individuals their earnings prospects
  ;============================================================================================================================================================================================
  ask individuals with [y_i = "NA"]
  [
    let ref_dat_low "NA"
    let ref_dat_high "NA"

    ifelse g_i = "male"
    [
      set ref_dat_low y_data_males_low
      set ref_dat_high y_data_males_high
    ]
    [
      set ref_dat_low y_data_females_low
      set ref_dat_high y_data_females_high
    ]

    let relevant_y_data "NA"


    ifelse s_i <  S_max
    [
      set relevant_y_data first filter [first ? = ref_year] ref_dat_low
    ]
    [
      set relevant_y_data first filter [first ? = ref_year] ref_dat_high
    ]

    let column 1

    let lower_boundary 0
    let upper_boundary lower_boundary + item 1 relevant_y_data
    let pick random-float (sum but-first relevant_y_data)

    while [y_i = "NA"]
    [
      ifelse (pick >= lower_boundary and pick < upper_boundary)
      [
        set y_i column
      ]
      [
        set column column + 1
        set lower_boundary upper_boundary
        set upper_boundary upper_boundary + item column relevant_y_data
      ]
    ]
  ]
end

to do_assign_individuals_a
  ;Assigns individuals' age
  ;============================================================================================================================================================================================
  ask individuals with [a_i = "NA"]
  [
    ifelse (set_up_done = TRUE)
    [
      set a_i 0
    ]
    [
      let col "NA"

      ifelse (g_i = "male")
      [
        set col 2
      ]
      [
        set col 1
      ]

      let row 0
      let lower_boundary 0
      let upper_boundary lower_boundary + item col item row ini_age_structure_data
      let pick random-float 1

      while [a_i = "NA"]
      [
        ifelse (pick >= lower_boundary and pick < upper_boundary or row = 110)
        [
          set a_i row * 10
        ]
        [
          set row row + 1
          set lower_boundary upper_boundary
          set upper_boundary upper_boundary + item col item row ini_age_structure_data
        ]
      ]
    ]
  ]
end

to do_update_school_enrolment_status
  ;Determines individuals' current school enrolment status, based on their current age and ultimate educational attainment
  ;============================================================================================================================================================================================

  ;Determine who has left school
  ask individuals with [r_i != 5]
  [
    if s_i = 1 and a_i >=  a_end_s_1 [set r_i 5]
    if s_i = 2 and a_i >=  a_end_s_2 [set r_i 5]
    if s_i = 3 and a_i >=  a_end_s_3 [set r_i 5]
    if s_i = 4 and a_i >=  a_end_s_4 [set r_i 5]
  ]

  ;Determine who transits to the next level
  ask individuals with [r_i != 5]
  [
    if a_i <  a_begin_s_2                        [set r_i 1]
    if a_i >= a_begin_s_2 and a_i < a_begin_s_3  [set r_i 2]
    if a_i >= a_begin_s_3 and a_i < a_begin_s_4  [set r_i 3]
    if a_i >= a_begin_s_4                        [set r_i 4]
  ]
end

to do_create_file_name_master
  ;Creates a master file name that stores information about the condition and run number, plus a random identifying number that makes it possible to differentiate runs
  ;============================================================================================================================================================================================
  set random_id random(10000000)

  set file_name_master country
  set file_name_master word file_name_master "_c_"
  set file_name_master word file_name_master condition_no
  set file_name_master word file_name_master "_id_"
  set file_name_master word file_name_master random_id
  set file_name_master word file_name_master ".txt"

  set individual_file word "output individual level\\ind_" file_name_master
  set marriage_file   word "output marriage level\\mar_" file_name_master

  while [file-exists? individual_file or file-exists? marriage_file]
  [
    set random_id random(10000000)
    set file_name_master country
    set file_name_master word file_name_master "_c_"
    set file_name_master word file_name_master condition_no
    set file_name_master word file_name_master "_id_"
    set file_name_master word file_name_master random_id
    set file_name_master word file_name_master ".txt"

    set individual_file word "output individual level\\ind_" file_name_master
    set marriage_file   word "output marriage level\\mar_" file_name_master
  ]
end

to do_initialize_individual_file
  ;Initializes file that stores census data about individuals
  ;============================================================================================================================================================================================
  file-open individual_file
  file-print (word "seed_to_use: " seed_to_use)
  file-print (word "my-seed: " my-seed)
  file-print (word "burn_in_years: " burn_in_years)
  file-print (word "start_year: " start_year)
  file-print (word "end_year: " end_year)
  file-print (word "steps_per_year: " steps_per_year)
  file-print (word "pop_size: " pop_size)
  file-print (word "S_max: " S_max)
  file-print (word "Y_max: " Y_max)
  file-print (word "A_max: " A_max)
  file-print (word "A_mar: " A_mar)
  file-print (word "wa_m: " wa_m)
  file-print (word "wa_f: " wa_f)
  file-print (word "wy_m: " wy_m)
  file-print (word "wy_f: " wy_f)
  file-print (word "ws_m: " ws_m)
  file-print (word "ws_f: " ws_f)
  file-print (word "delta: " delta)
  file-print (word "sigma_m: " sigma_m)
  file-print (word "sigma_f: " sigma_f)
  file-print (word "beta_m: " beta_m)
  file-print (word "beta_f: " beta_f)
  file-print (word "alpha_a_m: " alpha_a_m)
  file-print (word "alpha_a_f: " alpha_a_f)
  file-print (word "a_begin_s_2: " a_begin_s_2)
  file-print (word "a_begin_s_3: " a_begin_s_3)
  file-print (word "a_begin_s_4: " a_begin_s_4)
  file-print (word "a_end_s_1: " a_end_s_1)
  file-print (word "a_end_s_2: " a_end_s_2)
  file-print (word "a_end_s_3: " a_end_s_3)
  file-print (word "a_end_s_4: " a_end_s_4)
  file-print (word "run_id: " individual_file)
  file-print (word "=========================================")
  file-print (word "individual_i year_of_birth_i g_i s_i y_i a_i l_i n_relations n_marriages n_divorces n_widowed individual_j year_of_birth_j g_j s_j y_j a_j l_j n_relations_j n_marriages_j n_divorces_j n_widowed_j country year wa_m wa_f wy_m wy_f ws_m ws_f delta random_id condition_no")
  file-close
end

to do_initialize_marriage_file
  ;Initializes file that stores census data about marriages
  ;============================================================================================================================================================================================
  file-open marriage_file
  file-print (word "seed_to_use: " seed_to_use)
  file-print (word "my-seed: " my-seed)
  file-print (word "burn_in_years: " burn_in_years)
  file-print (word "start_year: " start_year)
  file-print (word "end_year: " end_year)
  file-print (word "steps_per_year: " steps_per_year)
  file-print (word "pop_size: " pop_size)
  file-print (word "S_max: " S_max)
  file-print (word "Y_max: " Y_max)
  file-print (word "A_max: " A_max)
  file-print (word "A_mar: " A_mar)
  file-print (word "wa_m: " wa_m)
  file-print (word "wa_f: " wa_f)
  file-print (word "wy_m: " wy_m)
  file-print (word "wy_f: " wy_f)
  file-print (word "ws_m: " ws_m)
  file-print (word "ws_f: " ws_f)
  file-print (word "delta: " delta)
  file-print (word "sigma_m: " sigma_m)
  file-print (word "sigma_f: " sigma_f)
  file-print (word "beta_m: " beta_m)
  file-print (word "beta_f: " beta_f)
  file-print (word "alpha_a_m: " alpha_a_m)
  file-print (word "alpha_a_f: " alpha_a_f)
  file-print (word "a_begin_s_2: " a_begin_s_2)
  file-print (word "a_begin_s_3: " a_begin_s_3)
  file-print (word "a_begin_s_4: " a_begin_s_4)
  file-print (word "a_end_s_1: " a_end_s_1)
  file-print (word "a_end_s_2: " a_end_s_2)
  file-print (word "a_end_s_3: " a_end_s_3)
  file-print (word "a_end_s_4: " a_end_s_4)
  file-print (word "run_id: " marriage_file)
  file-print (word "=========================================")
  file-print (word "m_id m_s m_y m_a m_marriage_order f_id f_s f_y f_a f_marriage_order year_marriage year_dissolution marriage_duration dissolution_reason dissolution_initiator country year wa_m wa_f wy_m wy_f ws_m ws_f delta random_id condition_no year_measured")
  file-close
end

;==============================================================================================================================================================================================
;Simulation procedure =========================================================================================================================================================================
;==============================================================================================================================================================================================
to go
  ;Defines the main simulation process
  ;============================================================================================================================================================================================

  ;Fertility and mortality happen at the beginning of each year
  ;------------------------------------------------------------------
  do_update_year
  do_fertility
  do_assign_individuals_s
  do_assign_individuals_y
  do_assign_individuals_a
  do_update_school_enrolment_status
  do_mortality

  ;The meeting, dating, divorce, marriage, and aging cycle within a given simulation year
  ;------------------------------------------------------------------
  repeat steps_per_year
  [
    do_update_p_seek_i
    do_meeting_dating_and_divorce
    do_marriage_proposals
    do_form_marriages
    do_update_age
    do_update_relation_times
    tick
  ]

  ;Determine whether individual census should be recorded in the current year
  ;------------------------------------------------------------------
  if (member? year measurement_points_ind = TRUE)
  [
    if write_to_file_individual = TRUE [do_write_individual_census]
  ]

  ;Determine whether marriage census should be recorded in the current year
  ;------------------------------------------------------------------
  if (member? year measurement_points_mar = TRUE)
  [
    if write_to_file_marriage = TRUE [do_write_marriage_census]
  ]

  if year = end_year[stop]
end

to do_update_p_seek_i
  ;Updates the probability that individual i will seek out somebody in the current time step
  ;============================================================================================================================================================================================
  ask individuals with [a_i >= A_mar]
  [
    set p_seek_i exp(- 1 * c_i * beta_i)
  ]
end

to do_meeting_dating_and_divorce
  ;Emulates the meeting, dating, and divorce process.
  ;For each individual (one by one), it is determined whether he/she will meet somebody and, if so, who this will be (i.e. somebody with the same or a different school enrolment status).
  ;Subsequently, both individuals need to decide whether they want to start dating each other.
  ;If both want to date, they leave/divorce possible current partners and a dating relation is created between them
  ;============================================================================================================================================================================================

  ;To speed up the simulation procedure, determine sets of individuals who are eligible for dating in this time step
  ;------------------------------------------------------------------
  let male_eligibles_r1 individuals with [g_i = "male" and a_i >= A_mar and r_i = 1]
  let male_eligibles_r2 individuals with [g_i = "male" and a_i >= A_mar and r_i = 2]
  let male_eligibles_r3 individuals with [g_i = "male" and a_i >= A_mar and r_i = 3]
  let male_eligibles_r4 individuals with [g_i = "male" and a_i >= A_mar and r_i = 4]
  let male_eligibles_r5 individuals with [g_i = "male" and a_i >= A_mar and r_i = 5]

  let male_eligibles_butr1 individuals with [g_i = "male" and a_i >= A_mar and r_i != 1]
  let male_eligibles_butr2 individuals with [g_i = "male" and a_i >= A_mar and r_i != 2]
  let male_eligibles_butr3 individuals with [g_i = "male" and a_i >= A_mar and r_i != 3]
  let male_eligibles_butr4 individuals with [g_i = "male" and a_i >= A_mar and r_i != 4]
  let male_eligibles_butr5 individuals with [g_i = "male" and a_i >= A_mar and r_i != 5]

  let female_eligibles_r1 individuals with [g_i = "female" and a_i >= A_mar and r_i = 1]
  let female_eligibles_r2 individuals with [g_i = "female" and a_i >= A_mar and r_i = 2]
  let female_eligibles_r3 individuals with [g_i = "female" and a_i >= A_mar and r_i = 3]
  let female_eligibles_r4 individuals with [g_i = "female" and a_i >= A_mar and r_i = 4]
  let female_eligibles_r5 individuals with [g_i = "female" and a_i >= A_mar and r_i = 5]

  let female_eligibles_butr1 individuals with [g_i = "female" and a_i >= A_mar and r_i != 1]
  let female_eligibles_butr2 individuals with [g_i = "female" and a_i >= A_mar and r_i != 2]
  let female_eligibles_butr3 individuals with [g_i = "female" and a_i >= A_mar and r_i != 3]
  let female_eligibles_butr4 individuals with [g_i = "female" and a_i >= A_mar and r_i != 4]
  let female_eligibles_butr5 individuals with [g_i = "female" and a_i >= A_mar and r_i != 5]

  ask individuals with [(a_i >= A_mar) and (p_seek_i > 0)]
  [
    ;Determine whether individual i meets somebody and, if so, whether the other individual will have the same or a different school enrolment status
    ;------------------------------------------------------------------
    if (random-float (1) < p_seek_i)
    [
      let ego who
      let alternative nobody

      ifelse (g_i = "male")
      [
        ifelse (random-float (1) < delta)
        [
          if ([r_i] of individual ego = 1 and female_eligibles_r1 != nobody)[set alternative one-of female_eligibles_r1]
          if ([r_i] of individual ego = 2 and female_eligibles_r2 != nobody)[set alternative one-of female_eligibles_r2]
          if ([r_i] of individual ego = 3 and female_eligibles_r3 != nobody)[set alternative one-of female_eligibles_r3]
          if ([r_i] of individual ego = 4 and female_eligibles_r4 != nobody)[set alternative one-of female_eligibles_r4]
          if ([r_i] of individual ego = 5 and female_eligibles_r5 != nobody)[set alternative one-of female_eligibles_r5]
        ]
        [
          if ([r_i] of individual ego = 1 and female_eligibles_butr1 != nobody)[set alternative one-of female_eligibles_butr1]
          if ([r_i] of individual ego = 2 and female_eligibles_butr2 != nobody)[set alternative one-of female_eligibles_butr2]
          if ([r_i] of individual ego = 3 and female_eligibles_butr3 != nobody)[set alternative one-of female_eligibles_butr3]
          if ([r_i] of individual ego = 4 and female_eligibles_butr4 != nobody)[set alternative one-of female_eligibles_butr4]
          if ([r_i] of individual ego = 5 and female_eligibles_butr5 != nobody)[set alternative one-of female_eligibles_butr5]
        ]
      ]
      [
        ifelse (random-float (1) < delta)
        [
          if ([r_i] of individual ego = 1 and male_eligibles_r1 != nobody)[set alternative one-of male_eligibles_r1]
          if ([r_i] of individual ego = 2 and male_eligibles_r2 != nobody)[set alternative one-of male_eligibles_r2]
          if ([r_i] of individual ego = 3 and male_eligibles_r3 != nobody)[set alternative one-of male_eligibles_r3]
          if ([r_i] of individual ego = 4 and male_eligibles_r4 != nobody)[set alternative one-of male_eligibles_r4]
          if ([r_i] of individual ego = 5 and male_eligibles_r5 != nobody)[set alternative one-of male_eligibles_r5]
        ]
        [
          if ([r_i] of individual ego = 1 and male_eligibles_butr1 != nobody)[set alternative one-of male_eligibles_butr1]
          if ([r_i] of individual ego = 2 and male_eligibles_butr2 != nobody)[set alternative one-of male_eligibles_butr2]
          if ([r_i] of individual ego = 3 and male_eligibles_butr3 != nobody)[set alternative one-of male_eligibles_butr3]
          if ([r_i] of individual ego = 4 and male_eligibles_butr4 != nobody)[set alternative one-of male_eligibles_butr4]
          if ([r_i] of individual ego = 5 and male_eligibles_butr5 != nobody)[set alternative one-of male_eligibles_butr5]
        ]
      ]

      ;Check whether the selected alternative accidentally is already the partner of individual i
      ;------------------------------------------------------------------
      if (alternative != nobody)
      [
        ask alternative
        [
          set relation_check 0
          if (link-neighbor? individual ego = true)[set relation_check 1]
        ]
        if ([relation_check] of alternative = 1)[set alternative nobody]
      ]

      ;If individual i meets somebody, determine whether individual i wants to date the other
      ;------------------------------------------------------------------
      if alternative != nobody
      [
        let dating_proposal 0
        set alternative [who] of alternative

        ifelse (g_i = "male")
        [
          ifelse (l_i ="single" or l_i = "divorced")
          [
            let vij (((( S_max - abs([s_i] of individual ego - [s_i] of individual alternative)) /   S_max) ^ [ws_i] of individual ego) * (((A_max  - abs([a_i] of individual alternative  - alpha_a_m)) / A_max) ^ [wa_i] of individual ego) * (([y_i] of individual alternative / Y_max) ^ [wy_i] of individual ego))

            if random-float (1) < (1 - exp(- 1 * [a_i] of individual ego * [sigma_i] of individual ego * vij))
            [
              set dating_proposal dating_proposal + 1
            ]
          ]
          [
            let partner [who] of one-of link-neighbors

            let vijp (((( S_max - abs([s_i] of individual ego - [s_i] of individual partner    )) /   S_max)  ^ [ws_i] of individual ego) * (((A_max - abs([a_i] of individual partner     - alpha_a_m)) / A_max) ^ [wa_i] of individual ego) * (([y_i] of individual partner     / Y_max) ^ [wy_i] of individual ego))
            let vija (((( S_max - abs([s_i] of individual ego - [s_i] of individual alternative)) /   S_max)  ^ [ws_i] of individual ego) * (((A_max - abs([a_i] of individual alternative - alpha_a_m)) / A_max) ^ [wa_i] of individual ego) * (([y_i] of individual alternative / Y_max) ^ [wy_i] of individual ego))

            if(vija > vijp)
            [
              if random-float (1) < (1 - exp(- 1 * ([a_i] of individual ego * [sigma_i] of individual ego * vija))) * exp(- 1 * ([c_i] of individual ego * [beta_i] of individual ego))
              [
                set dating_proposal dating_proposal + 1
              ]
            ]
          ]
        ]
        [
          ifelse (l_i ="single" or l_i = "divorced")
          [
            let vij (((( S_max - abs([s_i] of individual ego - [s_i] of individual alternative)) /   S_max) ^ [ws_i] of individual ego) * (((A_max - abs([a_i] of individual alternative  - ([a_i] of individual ego + alpha_a_f))) /  A_max) ^ [wa_i] of individual ego) * (([y_i] of individual alternative / Y_max) ^ [wy_i] of individual ego))

            if random-float (1) < (1 - exp(- 1 * [a_i] of individual ego * [sigma_i] of individual ego * vij))
            [
              set dating_proposal dating_proposal + 1
            ]
          ]
          [
            let partner [who] of one-of link-neighbors

            let vijp (((( S_max - abs([s_i] of individual ego - [s_i] of individual partner    )) /   S_max)  ^ [ws_i] of individual ego) * (((A_max - abs([a_i] of individual partner      - ([a_i] of individual ego + alpha_a_f))) /  A_max) ^ [wa_i] of individual ego) * (([y_i] of individual partner     / Y_max) ^ [wy_i] of individual ego))
            let vija (((( S_max - abs([s_i] of individual ego - [s_i] of individual alternative)) /   S_max)  ^ [ws_i] of individual ego) * (((A_max - abs([a_i] of individual alternative  - ([a_i] of individual ego + alpha_a_f))) /  A_max) ^ [wa_i] of individual ego) * (([y_i] of individual alternative / Y_max) ^ [wy_i] of individual ego))

            if(vija > vijp)
            [
              if random-float (1) < (1 - exp(- 1 * ([a_i] of individual ego * [sigma_i] of individual ego * vija))) * exp(- 1 * ([c_i] of individual ego * [beta_i] of individual ego))
              [
                set dating_proposal dating_proposal + 1
              ]
            ]
          ]
        ]

        ;If individual i wants to date the other individual, determine whether the other individual wants to date individual i
        ;------------------------------------------------------------------
        if (dating_proposal = 1)
        [
          ask individual alternative
          [
            ifelse (g_i = "male")
            [
              ifelse (l_i ="single" or l_i = "divorced")
              [
                let vij (((( S_max - abs([s_i] of individual ego  - [s_i] of individual alternative)) /   S_max) ^ [ws_i] of individual alternative) * (((A_max - abs([a_i] of individual ego - alpha_a_m)) / A_max) ^ [wa_i] of individual alternative) * (([y_i] of individual ego / Y_max) ^ [wy_i] of individual alternative))

                if random-float (1) < (1 - exp(- 1 * [a_i] of individual alternative * [sigma_i] of individual alternative * vij))
                [
                  set dating_proposal dating_proposal + 1
                ]
              ]
              [
                let partner [who] of one-of link-neighbors

                let vijp (((( S_max - abs([s_i] of individual alternative - [s_i] of individual partner)) /   S_max) ^  [ws_i] of individual alternative) * (((A_max - abs([a_i] of individual partner - alpha_a_m)) / A_max) ^ [wa_i] of individual alternative)  * (([y_i] of individual partner / Y_max) ^ [wy_i] of individual alternative))
                let vija (((( S_max - abs([s_i] of individual alternative - [s_i] of individual ego    )) /   S_max) ^  [ws_i] of individual alternative) * (((A_max - abs([a_i] of individual ego     - alpha_a_m)) / A_max) ^ [wa_i] of individual alternative)  * (([y_i] of individual ego     / Y_max) ^ [wy_i] of individual alternative))

                if(vija > vijp)
                  [
                    if random-float (1) < (1 - exp(- 1 * ([a_i] of individual alternative * [sigma_i] of individual alternative * vija))) * exp(- 1 * ([c_i] of individual alternative * [beta_i] of individual alternative))
                    [
                      set dating_proposal dating_proposal + 1
                    ]
                  ]
              ]
            ]
            [
              ifelse (l_i ="single" or l_i = "divorced")
              [
                let vij (((( S_max - abs([s_i] of individual ego  - [s_i] of individual alternative)) /   S_max) ^ [ws_i] of individual alternative) * (((A_max - abs([a_i] of individual ego - ([a_i] of individual alternative + alpha_a_f))) /  A_max) ^ [wa_i] of individual alternative) * (([y_i] of individual ego / Y_max) ^ [wy_i] of individual alternative))

                if random-float (1) < (1 - exp(- 1 * [a_i] of individual alternative * [sigma_i] of individual alternative * vij))
                [
                  set dating_proposal dating_proposal + 1
                ]
              ]
              [
                let partner [who] of one-of link-neighbors

                let vijp (((( S_max - abs([s_i] of individual alternative - [s_i] of individual partner)) /   S_max) ^  [ws_i] of individual alternative) * (((A_max - abs([a_i] of individual partner  - ([a_i] of individual alternative + alpha_a_f))) /  A_max) ^ [wa_i] of individual alternative)  * (([y_i] of individual partner / Y_max) ^ [wy_i] of individual alternative))
                let vija (((( S_max - abs([s_i] of individual alternative - [s_i] of individual ego    )) /   S_max) ^  [ws_i] of individual alternative) * (((A_max - abs([a_i] of individual ego      - ([a_i] of individual alternative + alpha_a_f))) /  A_max) ^ [wa_i] of individual alternative)  * (([y_i] of individual ego     / Y_max) ^ [wy_i] of individual alternative))

                if(vija > vijp)
                [
                  if random-float (1) < (1 - exp(- 1 * ([a_i] of individual alternative * [sigma_i] of individual alternative * vija))) * exp(- 1 * ([c_i] of individual alternative * [beta_i] of individual alternative))
                  [
                    set dating_proposal dating_proposal + 1
                  ]
                ]
              ]
            ]
          ]
        ]

        ;If both individuals want to date, let them leave/divorce any current partners and create dating relation between them
        ;------------------------------------------------------------------
        if (dating_proposal = 2)
        [
          ask individual ego
          [
            if l_i = "married"
            [
              set n_divorces n_divorces + 1
              ask marriage_records with [marriage_id = [current_marriage_id] of individual ego]
              [
                set year_dissolution year
                set marriage_duration [c_i] of individual ego
                set dissolution_reason "divorce"
                set dissolution_initiator [g_i] of individual ego
              ]
              set current_marriage_id "NA"
            ]

            set l_i "dating"
            set c_i 0
            set n_relations n_relations + 1
            set marriage_proposal nobody

            if count link-neighbors > 0
            [
              ask link-neighbors
              [
                ifelse l_i = "married"
                [
                  set n_divorces n_divorces + 1
                  set current_marriage_id "NA"
                  set l_i "divorced"
                ]
                [
                  set l_i "single"
                ]
                set marriage_proposal nobody
                set c_i 0
                ask link-with individual ego [die]
              ]
            ]
          ]

          ask individual alternative
          [
            if l_i = "married"
            [
              set n_divorces n_divorces + 1
              ask marriage_records with [marriage_id = [current_marriage_id] of individual alternative]
              [
                set year_dissolution year
                set marriage_duration [c_i] of individual alternative
                set dissolution_reason "divorce"
                set dissolution_initiator [g_i] of individual alternative
              ]
              set current_marriage_id "NA"
            ]

            set l_i "dating"
            set c_i 0
            set n_relations n_relations + 1
            set marriage_proposal nobody

            if count link-neighbors > 0
            [
              ask link-neighbors
              [
                ifelse l_i = "married"
                [
                  set n_divorces n_divorces + 1
                  set current_marriage_id "NA"
                  set l_i "divorced"
                ]
                [
                  set l_i "single"
                ]
                set marriage_proposal nobody
                set c_i 0
                ask link-with individual alternative [die]
              ]
            ]
            create-dating_relation-with individual ego [set color 7]
          ]
        ]
      ]
    ]
  ]
end

to do_marriage_proposals
  ;The marriage decision process
  ;============================================================================================================================================================================================
  ask individuals with [(a_i >= A_mar) and (count dating_relation-neighbors > 0)]
  [
    let ego who
    let partner [who] of one-of dating_relation-neighbors

    let vij "NA"

    ifelse (g_i = "male")
    [
      set vij (((( S_max - abs([s_i] of individual ego - [s_i] of individual partner)) /   S_max) ^ [ws_i] of individual ego) * (((A_max - abs([a_i] of individual partner - alpha_a_m)) / A_max) ^ [wa_i] of individual ego) * (([y_i] of individual partner / Y_max) ^ [wy_i] of individual ego))
    ]
    [
      set vij (((( S_max - abs([s_i] of individual ego - [s_i] of individual partner)) /   S_max) ^ [ws_i] of individual ego) * (((A_max - abs([a_i] of individual partner - ([a_i] of individual ego + alpha_a_f))) /  A_max) ^ [wa_i] of individual ego) * (([y_i] of individual partner / Y_max) ^ [wy_i] of individual ego))
    ]

    if random-float (1) < (1 - exp(- 1 * ([a_i] of individual ego * [sigma_i] of individual ego * vij))) * (1 - exp(- 1 * ([c_i] of individual ego * [beta_i] of individual ego)))
    [
      set marriage_proposal partner
    ]
  ]
end

to do_form_marriages
  ;Creates a marriage relation between individuals who both propose to marry each other and creates a marriage object that traces the characteristics of the marriage
  ;============================================================================================================================================================================================
  ask dating_relations with [([marriage_proposal] of end1 = [who] of end2) and ([marriage_proposal] of end2 = [who] of end1)]
  [
    let E1 [who] of end1
    let E2 [who] of end2

    ask both-ends
    [
      set l_i "married"
      set n_marriages n_marriages + 1
      ifelse ([g_i] of individual E1 = "male")
      [
        set current_marriage_id (word [who] of individual E1 "_" [who] of individual E2)
      ]
      [
        set current_marriage_id (word [who] of individual E2 "_" [who] of individual E1)
      ]

    ]
    set breed marriage_relations
    set color 15

    ask individual E1
    [
      hatch-marriage_records 1
      [
        set hidden? true

        ifelse ([g_i] of individual E1 = "male")
        [
          set m_id [who] of individual E1
          set m_s  [s_i] of individual E1
          set m_y  [y_i] of individual E1
          set m_a  [a_i] of individual E1
          set m_marriage_order [n_marriages] of individual E1

          set f_id [who] of individual E2
          set f_s  [s_i] of individual E2
          set f_y  [y_i] of individual E2
          set f_a  [a_i] of individual E2
          set f_marriage_order [n_marriages] of individual E2

          set marriage_id (word [who] of individual E1 "_" [who] of individual E2)
        ]
        [
          set m_id [who] of individual E2
          set m_s  [s_i] of individual E2
          set m_y  [y_i] of individual E2
          set m_a  [a_i] of individual E2
          set m_marriage_order [n_marriages] of individual E2

          set f_id [who] of individual E1
          set f_s  [s_i] of individual E1
          set f_y  [y_i] of individual E1
          set f_a  [a_i] of individual E1
          set f_marriage_order [n_marriages] of individual E1

          set marriage_id (word [who] of individual E2 "_" [who] of individual E1)
        ]
        set year_marriage year
        set year_dissolution "NA"
        set marriage_duration "NA"
        set dissolution_reason "NA"
        set dissolution_initiator "NA"
      ]
    ]
  ]
end

to do_update_relation_times
  ;Updates the relationship times of individuals who are dating or married
  ;============================================================================================================================================================================================
  ask individuals with [l_i != "single" and l_i != "divorced"]
  [
    set c_i c_i + 1
  ]
end

to do_update_age
  ;Updates individuals' age
  ;============================================================================================================================================================================================
  ask individuals
  [
    set a_i a_i + 1
  ]
end

to do_fertility
  ;Simulates fertility for each female individual based on period- and age-specific fertility rates
  ;============================================================================================================================================================================================
  set asfr_current filter [first ? = ref_year] asfr_data

  let counter 0

  while [counter <= (55 - 12)]
  [
    ask individuals with [g_i = "female" and a_i != "NA" and floor(a_i / 10) = item 1 item counter asfr_current]
    [
      if random-float (1) < item 2 item counter asfr_current
      [
        ;Create male or female individual
        ;------------------------------------------------------------------
        ifelse random-float (1) < .512
        [
          hatch-individuals 1
          [
            set g_i "male"
            set s_i "NA"
            set y_i "NA"
            set a_i "NA"
            set r_i "NA"
            set shape "triangle"
            set color blue
            set sigma_i sigma_m
            set beta_i beta_m
            set wa_i wa_m
            set ws_i ws_m
            set wy_i wy_m
            set c_i 0
            set l_i "single"
            set p_seek_i 0
            set year_of_birth year
            set marriage_proposal nobody
            set n_relations 0
            set n_marriages 0
            set n_divorces 0
            set n_widowed 0
            set marriage_proposal nobody
            set current_marriage_id "NA"
            setxy random-xcor random-ycor
            set label who
          ]
        ]
        [
          hatch-individuals 1
          [
            set g_i "female"
            set s_i "NA"
            set y_i "NA"
            set a_i "NA"
            set r_i "NA"
            set shape "circle"
            set color red
            set sigma_i sigma_f
            set beta_i beta_f
            set wa_i wa_f
            set ws_i ws_f
            set wy_i wy_f
            set c_i 0
            set l_i "single"
            set p_seek_i 0
            set year_of_birth year
            set marriage_proposal nobody
            set n_relations 0
            set n_marriages 0
            set n_divorces 0
            set n_widowed 0
            set marriage_proposal nobody
            set current_marriage_id "NA"
            setxy random-xcor random-ycor
            set label who
          ]
        ]
      ]
    ]
    set counter counter + 1
  ]
end

to do_mortality
  ;For each agent, there is a chance that he/she dies based on period-, gender-, and age-specific mortality rates
  ;============================================================================================================================================================================================
  set d_current filter [first ? = ref_year] d_data

  let counter 0

  while [counter <= 110]
  [
    ask individuals with [a_i != "NA" and floor(a_i / 10) = item 1 item counter d_current]
    [
      let col "NA"

      ifelse (g_i = "male")
      [
        set col 3
      ]
      [
        set col 2
      ]

      if random-float (1) <  item col item counter d_current
      [
        if count link-neighbors > 0
        [
          ask one-of link-neighbors
          [
            let E1 who
            if l_i = "married"
            [
              set n_widowed n_widowed + 1
              ask marriage_records with [marriage_id = [current_marriage_id] of individual E1]
              [
                set year_dissolution year
                set marriage_duration [c_i] of individual E1
                set dissolution_reason "death"
              ]
              set current_marriage_id "NA"
            ]

            set l_i "single"
            set marriage_proposal nobody
            set c_i 0
          ]
        ]
        die
      ]
    ]
    set counter counter + 1
  ]
end

to do_write_individual_census
  ;Takes a census of individuals
  ;============================================================================================================================================================================================
  file-close
  file-open individual_file

  ;Write the individual census to the output file if the output has been activated
  ;------------------------------------------------------------------
  ask individuals with [l_i != "single" and l_i != "divorced"]
  [
    file-print (word who " " year_of_birth " " g_i " " s_i " " y_i " " a_i " " l_i " " n_relations " " n_marriages " " n_divorces " " n_widowed " " [who] of one-of link-neighbors " " [year_of_birth] of one-of link-neighbors " " [g_i] of one-of link-neighbors " " [s_i] of one-of link-neighbors " "[y_i] of one-of link-neighbors " " [a_i] of one-of link-neighbors " " [l_i] of one-of link-neighbors " " [n_relations] of one-of link-neighbors " " [n_marriages] of one-of link-neighbors " " [n_divorces] of one-of link-neighbors " " [n_widowed] of one-of link-neighbors " " country " " year " " wa_m " " wa_f " " wy_m " " wy_f " " ws_m " " ws_f " " delta " " random_id " " condition_no)
  ]
  ask individuals with [l_i = "single" or l_i = "divorced"]
  [
    file-print (word who " " year_of_birth " " g_i " " s_i " " y_i " " a_i  " " l_i " " n_relations " " n_marriages " " n_divorces " " n_widowed " NA NA NA NA NA NA NA NA NA NA NA " country " " year " " wa_m " " wa_f " " wy_m " " wy_f " " ws_m " " ws_f " " delta " " random_id " " condition_no)
  ]

  ;Make sure file is closed after everything has been written
  ;------------------------------------------------------------------
  file-close
end

to do_write_marriage_census
  ;Takes a census of marriages
  ;============================================================================================================================================================================================
  file-close
  file-open marriage_file

  ask marriage_records with [year_marriage > 0]
  [
    file-print (word m_id " " m_s " " m_y " " m_a " " m_marriage_order " " f_id " " f_s " " f_y " " f_a " " f_marriage_order " " year_marriage " " year_dissolution " " marriage_duration " " dissolution_reason " " dissolution_initiator " " country " " year " " wa_m " " wa_f " " wy_m " " wy_f " " ws_m " " ws_f " " delta " " random_id " " condition_no " " year)
  ]

  ;Make sure file is closed after everything has been written
  ;------------------------------------------------------------------
  file-close
end
@#$#@#$#@
GRAPHICS-WINDOW
1021
62
1400
462
16
16
11.2
1
10
1
1
1
0
0
0
1
-16
16
-16
16
1
1
1
ticks
30.0

BUTTON
11
11
86
44
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
11
50
86
83
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

CHOOSER
12
126
135
171
seed_to_use
seed_to_use
"random" "own"
0

INPUTBOX
13
219
134
279
own_seed
2147483647
1
0
Number

INPUTBOX
14
564
137
624
end_year
2064
1
0
Number

INPUTBOX
14
435
137
495
burn_in_years
60
1
0
Number

SWITCH
13
290
182
323
write_to_file_individual
write_to_file_individual
0
1
-1000

INPUTBOX
318
267
441
327
A_mar
160
1
0
Number

CHOOSER
14
382
136
427
country
country
"BE" "DE" "DK" "ES" "FI" "FR" "GR" "IE" "NL" "PT" "SE" "UK"
11

INPUTBOX
317
129
440
189
steps_per_year
10
1
0
Number

INPUTBOX
14
501
137
561
start_year
1921
1
0
Number

INPUTBOX
725
261
822
321
sigma_m
0.0015
1
0
Number

INPUTBOX
612
263
717
323
sigma_f
0.0030
1
0
Number

INPUTBOX
725
199
822
259
beta_m
0.015
1
0
Number

INPUTBOX
612
200
717
260
beta_f
0.015
1
0
Number

INPUTBOX
723
450
821
510
wa_m
6.887
1
0
Number

INPUTBOX
613
452
718
512
wa_f
14.895
1
0
Number

INPUTBOX
724
324
821
384
ws_m
0.934
1
0
Number

INPUTBOX
613
327
718
387
ws_f
0.385
1
0
Number

INPUTBOX
723
388
821
448
wy_m
1.025
1
0
Number

INPUTBOX
613
389
718
449
wy_f
1.201
1
0
Number

INPUTBOX
614
516
718
576
alpha_a_f
25
1
0
Number

INPUTBOX
723
516
821
576
alpha_a_m
240
1
0
Number

INPUTBOX
612
133
716
193
delta
0.9
1
0
Number

INPUTBOX
240
10
330
70
condition_no
0
1
0
Number

INPUTBOX
14
858
133
918
a_begin_s_4
190
1
0
Number

INPUTBOX
293
664
413
724
a_end_s_1
60
1
0
Number

INPUTBOX
293
729
413
789
a_end_s_2
160
1
0
Number

INPUTBOX
293
858
413
918
a_end_s_4
240
1
0
Number

INPUTBOX
14
729
132
789
a_begin_s_2
60
1
0
Number

INPUTBOX
14
793
133
853
a_begin_s_3
100
1
0
Number

INPUTBOX
293
793
413
853
a_end_s_3
190
1
0
Number

TEXTBOX
143
383
290
411
Selection of country-level input data to be used.
11
0.0
1

TEXTBOX
93
12
243
30
Initialize a simulation run.
11
0.0
1

TEXTBOX
92
50
242
68
Start a simulation run.
11
0.0
1

TEXTBOX
142
125
292
209
Defines whether a random or a user-defined seed should be used for initializing pseudo random numbers. Using a user-defined seed allows the full replication of a given run.
11
0.0
1

TEXTBOX
142
219
292
237
User-defined seed.
11
0.0
1

TEXTBOX
188
292
316
320
Should individual-level results be recorded?
11
0.0
1

INPUTBOX
318
197
440
257
pop_size
1000
1
0
Number

TEXTBOX
447
197
595
239
Numbers of female/male individuals in the population.
11
0.0
1

TEXTBOX
145
436
295
506
Number of simulation years that should be disregarded before the main phase of a given simulation run begins.
11
0.0
1

TEXTBOX
144
502
294
544
Year in which the simulation begins after the burn-in phase.
11
0.0
1

TEXTBOX
143
566
293
608
Simulation year after which the simulation stops.
11
0.0
1

TEXTBOX
446
131
596
173
Number of times the partner search algorithm is run per simulation year.
11
0.0
1

TEXTBOX
447
269
597
297
Age at which individuals enter the marriage market.
11
0.0
1

TEXTBOX
448
332
598
388
Age after which individuals are removed from the marriage market (i.e. age at which they die at the latest).
11
0.0
1

TEXTBOX
138
729
288
757
Age at which individuals enter primary education.
11
0.0
1

TEXTBOX
138
795
288
823
Age at which individuals enter secondary education.
11
0.0
1

TEXTBOX
136
859
286
887
Age at which individuals enter tertiary education.
11
0.0
1

TEXTBOX
418
663
568
691
Age at which individuals leave preschool state.
11
0.0
1

TEXTBOX
420
730
570
772
Age at which individuals leave primary/lower secondary education.
11
0.0
1

TEXTBOX
421
793
571
821
Age at which individuals leave secondary education.
11
0.0
1

TEXTBOX
419
860
569
888
Age at which individuals leave tertiary education.
11
0.0
1

TEXTBOX
11
646
587
672
------------------------------------------------Transitions between educational levels----------------------------------------------
11
0.0
1

TEXTBOX
615
100
1031
128
------------------------Main factors during partner search--------------------------------
11
0.0
1

TEXTBOX
15
102
577
144
-------------------------------------------Basic properties of the simulation run-------------------------------------------
11
0.0
1

TEXTBOX
598
108
613
864
|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|\n|
11
0.0
1

TEXTBOX
721
133
871
161
Structuring effect of the educational system.
11
0.0
1

TEXTBOX
835
260
985
288
Age pressure factor for female/male individuals.
11
0.0
1

TEXTBOX
832
199
982
227
Commitment factor for female/male individuals.
11
0.0
1

TEXTBOX
832
325
982
367
Importance that female/male individuals attach to the education of partners.
11
0.0
1

TEXTBOX
832
385
982
441
Importance that female/male individuals attach to the earnings prospects of partners.
11
0.0
1

TEXTBOX
831
449
981
491
Importance that female/male individuals attach to the age of partners.
11
0.0
1

TEXTBOX
830
517
980
545
Age that female/male individuals prefer in a partner.
11
0.0
1

TEXTBOX
341
12
491
82
Auxiliary variable that stores information about the condition number in larger simulation experiments
11
0.0
1

TEXTBOX
1025
469
1403
539
Upon initialization, the view window shows the female (circles) and male (triangles) individuals in the population. Gray links between two individuals indicate that they are dating; red links indicate that they are married.
11
0.0
1

INPUTBOX
319
396
441
456
S_max
4
1
0
Number

INPUTBOX
319
462
441
522
Y_max
5
1
0
Number

TEXTBOX
449
399
599
427
Highest educational level that individuals can attain.
11
0.0
1

TEXTBOX
448
466
598
494
Highest earnings prospects that individuals can attain.
11
0.0
1

MONITOR
1022
11
1118
56
simulation year
year
17
1
11

TEXTBOX
1127
11
1277
53
Monitors the simulation year. The value is 0 during the burn-in phase.
11
0.0
1

SWITCH
14
335
182
368
write_to_file_marriage
write_to_file_marriage
0
1
-1000

TEXTBOX
187
337
314
365
Should marriage-level results be recorded?
11
0.0
1

INPUTBOX
318
333
441
393
A_max
1100
1
0
Number

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.2.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="250" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <enumeratedValueSet variable="a_begin_s_4">
      <value value="190"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="wy_m">
      <value value="1.025"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="alpha_a_f">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="beta_f">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="own_seed">
      <value value="2147483647"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="steps_per_year">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="a_end_s_2">
      <value value="160"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="wy_f">
      <value value="1.201"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="condition_no">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="pop_size">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ws_f">
      <value value="0.385"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="burn_in_years">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="a_end_s_4">
      <value value="240"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="country">
      <value value="&quot;UK&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="end_year">
      <value value="2064"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="seed_to_use">
      <value value="&quot;random&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="a_end_s_1">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="alpha_a_m">
      <value value="240"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Y_max">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="delta">
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="write_to_file_individual">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="start_year">
      <value value="1921"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sigma_m">
      <value value="0.0015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="wa_f">
      <value value="14.895"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="beta_m">
      <value value="0.015"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="A_mar">
      <value value="160"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="a_end_s_3">
      <value value="190"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="wa_m">
      <value value="6.887"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="A_max">
      <value value="1100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="a_begin_s_2">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="write_to_file_marriage">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="a_begin_s_3">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="S_max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="sigma_f">
      <value value="0.0030"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="ws_m">
      <value value="0.934"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
